#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""

Utilitaire de mise à jour des variables
pour les versions >= 2.4.1

"""
from creole3.upgrade import log
from pyeole.i18n import i18n
from tiramisu.setting import owners
_ = i18n('creole')

class Upgrade():
    """
    Méthodes pour la mise à niveau des variables
    """
    def __init__(self, config):
        owner = 'upgrade'
        if owner not in dir(owners):
            owners.addowner(owner)
        self.config = config
        self.owner = getattr(owners, owner)
        self.unknown_options = config.impl_get_information(u'unknown_options')

    def get_old_value(self, old_variable, default=None):
        """
        Retourne la valeur d'une variable "disparue"
        """
        try:
            return self.unknown_options[old_variable][u'val']
        except KeyError:
            return default

    def get_value(self, variable, default=None):
        """
        Retourne la valeur d'une variable "connue"
        """
        try:
            path = self.get_path(variable)
        except AttributeError:
            return default
        return self.config.getattr(path,
                                   force_permissive=True)

    def get_path(self, variable):
        """
        Retourne le chemin complet d'une variable
        """
        return self.config.find_first(byname=variable, type_='path')

    def modify_owner(self, path):
        """
        Modifie le propriétaire d'une variable
        """
        option = self.config.unwrap_from_path(path)
        self.config.cfgimpl_get_values().setowner(option,
                                                  self.owner)

    def set_value(self, variable, value):
        """
        Modifie la valeur d'une variable
        """
        path = self.get_path(variable)
        try:
            self.config._setattr(path, value,
                                 force_permissive=True)
            self.modify_owner(path)
            log.info(_(u"Variable updated: {0} = {1}").format(variable, value))
        except ValueError:
            option = self.config.unwrap_from_path(path)
            self.config.cfgimpl_get_settings()[option].append('load_error')

    def append_value(self, variable, value):
        """
        Ajoute une valeur à une variable multi
        """
        path = self.get_path(variable)
        multi = self.config.getattr(path,
                                    force_permissive=True)
        multi.append(value)
        self.modify_owner(path)

    def modify_last_value(self, variable, value):
        """
        Modifie la dernière valeur d'une variable multi
        """
        path = self.get_path(variable)
        multi = self.config.getattr(path,
                            force_permissive=True)
        multi[-1] = value
        self.modify_owner(path)

    def move(self, old_variable, new_variable):
        """
        Déplace les données d'une variable "disparue"
        vers une nouvelle variable
        """
        if old_variable in self.unknown_options:
            value = self.unknown_options[old_variable][u'val']
            self.set_value(new_variable, value)
            del(self.unknown_options[old_variable])
            log.info(_(u"Variable {0} has been renamed to {1}").format(old_variable, new_variable))

    def copy(self, old_variable, new_variable, only_if_modified=True):
        """
        Copie la valeur d'une variable existante vers une autre
        Si la valeur "old" est une multi et pas la "new" => copie la 1er valeur de la liste
        Si la valeur "old" n'est pas  une multi et la "new" ne l'est pas => transforme la valeur en liste
        only_if_modified: si True ne copie que les valeurs qui sont modifiées
        """
        try:
            # si les deux variables existe => migration
            old_path = self.get_path(old_variable)
            new_path = self.get_path(new_variable)
        except AttributeError:
            pass
        else:
            old_option = self.config.unwrap_from_path(old_path)
            new_option = self.config.unwrap_from_path(new_path)
            # si la nouvelle option n'est pas modifié et si la valeur est modifié ou only_if_modified est False
            if self.config.cfgimpl_get_values().is_default_owner(new_option) and \
                    (not only_if_modified or
                     not self.config.cfgimpl_get_values().is_default_owner(old_option)):
                old_value = self.config.getattr(old_path,
                                                force_permissive=True)
                if old_option.impl_is_multi() and not new_option.impl_is_multi():
                    if len(old_value) != 0:
                        old_value = old_value[0]
                    else:
                        old_value = None
                if not old_option.impl_is_multi() and new_option.impl_is_multi():
                    if old_value is None:
                        old_value = []
                    else:
                        old_value = [old_value]
                self.set_value(new_variable, old_value)


class Upgrade240_241(Upgrade):
    """
    Mise à jour d'une configuration
    de 2.4.0 vers 2.4.1
    """
    def run(self):
        """
        Lancement des traitements
        """
        log.info(_(u"Starting {0} to {1} upgrade").format('2.4.0', '2.4.1'))

        # renommage des variables "era_proxy_bypass"
        for i in range(1, 5):
            self.move('era_proxy_bypass_eth{0}'.format(i), 'proxy_bypass_network_eth{0}'.format(i))

        # fusion des variables "proxy_bypass" et "wpad_exclude"
        if 'adresse_ip_wpad_exclude' in self.unknown_options:
            for idx, value in enumerate(self.get_old_value('adresse_ip_wpad_exclude')):
                interface = self.get_old_value('interface_wpad_exclude')[idx]
                if interface == 'Toutes':
                    interfaces = range(1, 5)
                elif int(interface) in range(1, 5):
                    interfaces = [interface]
                else:
                    log.error(_(u"Invalid value : {0} in old variable {1}").format(interface, 'interface_wpad_exclude'))
                    continue
                for i in interfaces:
                    self.append_value('proxy_bypass_network_eth{0}'.format(i), value)
                    self.modify_last_value('proxy_bypass_netmask_eth{0}'.format(i), self.get_old_value('adresse_netmask_wpad_exclude')[idx])
            del(self.unknown_options['adresse_ip_wpad_exclude'])
            del(self.unknown_options['adresse_netmask_wpad_exclude'])
            del(self.unknown_options['interface_wpad_exclude'])

        # passage à oui des variables "proxy_bypass_ethX" si nécessaire
        for i in range(1, 5):
            if len(self.get_value('proxy_bypass_network_eth{0}'.format(i), [])) > 0:
                self.set_value('proxy_bypass_eth{0}'.format(i), u'oui')

        # transfert des variables nom_domaine_wpad_exclude
        if 'nom_domaine_wpad_exclude' in self.unknown_options:
            for idx, value in enumerate(self.get_old_value('nom_domaine_wpad_exclude')):
                interface = self.get_old_value('nom_interface_wpad_exclude')[idx]
                if interface == 'Toutes':
                    interfaces = range(1, 5)
                elif int(interface) in range(1, 5):
                    interfaces = [interface]
                else:
                    log.error(_(u"Invalid value : {0} in old variable {1}").format(interface, 'nom_interface_wpad_exclude'))
                    continue
                for i in interfaces:
                    self.append_value('proxy_bypass_domain_eth{0}'.format(i), value)
            del(self.unknown_options['nom_domaine_wpad_exclude'])
            del(self.unknown_options['nom_interface_wpad_exclude'])

        # nom_serveur_scribe_dmz/ip_serveur_scribe_dmz => mandatory (#11713)
        if self.get_value('install_scribe_dmz') == u'oui':
            if self.get_value('nom_serveur_scribe_dmz') == None or self.get_value('ip_serveur_scribe_dmz') == None:
                self.set_value('install_scribe_dmz', u'non')


class Upgrade241_242(Upgrade):

    def run(self):
        log.info(_(u"Starting {0} to {1} upgrade").format('2.4.1', '2.4.2'))
        # migration des variables eolesso vers client LDAP #10821
        self.copy('eolesso_port_ldap', 'ldap_port')
        self.copy('eolesso_ldap_reader', 'ldap_reader')
        self.copy('eolesso_ldap_reader_passfile', 'ldap_reader_passfile')
        self.copy('eolesso_ldap_match_attribute', 'ldap_match_attribute')
        self.copy('eolesso_ldap_filter_user', 'ldap_filter_user')
        self.copy('eolesso_ldap_filter_group', 'ldap_filter_group')
        self.copy('eolesso_ldap_dntree_user', 'ldap_dntree_user')
        self.copy('eolesso_ldap_dntree_group', 'ldap_dntree_group')
        self.copy('eolesso_ldap_fill_displayname', 'ldap_fill_displayname')
        self.copy('eolesso_ldap_fill_mail', 'ldap_fill_mail')
        self.copy('eolesso_ldap_fill_fonction', 'ldap_fill_fonction')
        self.copy('eolesso_ldap_fill_categorie', 'ldap_fill_categorie')
        self.copy('eolesso_ldap_fill_rne', 'ldap_fill_rne')
        self.copy('eolesso_ldap_fill_fredurne', 'ldap_fill_fredurne')
        self.copy('eolesso_ldap_fill_displaygroup', 'ldap_fill_displaygroup')

        # migration des variables courier #10987
        courier_val = self.get_old_value('activer_courier')
        if courier_val is not None:
            if courier_val == 'non':
                self.set_value('activer_recuperation_courriel', 'non')
            elif not 'imap' in courier_val:
                self.set_value('activer_courier_imap', 'non')
            if 'pop' in courier_val:
                self.set_value('activer_courier_pop', 'oui')


def upgrade_0(config):
    """
    Lancement de la mise à niveau
    de 2.4.0 vers 2.4.1
    """
    upgrade = Upgrade240_241(config)
    upgrade.run()


def upgrade_1(config):
    """
    Lancement de la mise à niveau
    de 2.4.1 vers 2.4.2
    """
    upgrade = Upgrade241_242(config)
    upgrade.run()


def upgrade24(old_version, current_version, config):
    for i in xrange(int(old_version.split('.')[-1]), int(current_version.split('.')[-1])):
        func = 'upgrade_' + str(i)
        if func in globals():
            globals()[func](config)
